from __future__ import absolute_import
from __future__ import print_function
try:
    import numpy
    import reflex
    from pipeline_product import PipelineProduct
    import pipeline_display
    import matplotlib.gridspec as gridspec
    import espda_discrete_slider
    try:
       from astropy.io import fits as pyfits
    except ImportError:
       import pyfits 
    import_success = True

except ImportError:
    import_success = False
    print("Error importing modules pyfits, wx, matplotlib, numpy")

def paragraph(text, width=None):
    import textwrap
    if width is None:
        return textwrap.dedent(text).replace('\n', ' ').strip()
    else:
        return textwrap.fill(textwrap.dedent(text), width=width)

class DataPlotterManager:

    recipe_name = "espda_compu_eqwidth"
    line_index = 233


    def setInteractiveParameters(self):
        paramList = list()
#        paramList.append(reflex.RecipeParameter(recipe = self.recipe_name,
#            displayName = "wavel_min", group = "WRange", description =
#            "Starting wavelength of the spectrum [nm] [380-780]. [380.0]"))
#        paramList.append(reflex.RecipeParameter(recipe = self.recipe_name,
#            displayName = "wavel_max",  group = "WRange", description =
#                                                "Ending wavelength of the spectrum [nm]. range: [380,780]. [780]"))
        paramList.append(reflex.RecipeParameter(recipe = self.recipe_name,
            displayName = "space",      group = "Fitting", description =
            "Width of the local spectral region for each side of the line [nm]; range: [0.1,5]. [0.3][nm]; "))
        paramList.append(reflex.RecipeParameter(recipe = self.recipe_name,
            displayName = "ew_min",  group = "Thresholds", description =
            "Minimum value for a line EW to be accepted [mA]; 2.0 [0, 400]"))
        paramList.append(reflex.RecipeParameter(recipe = self.recipe_name,
            displayName = "cont_rejt",  group = "Thresholds", description =
           "Rejection threshold for the continuum determination." \
           "The default value depends on the signal-to-- noise (-1); " \
           " range: [0.5,1[. [-1.0]"))
        paramList.append(reflex.RecipeParameter(recipe = self.recipe_name,
            displayName = "cont_iter",   group = "Thresholds", description =
            "Number of iteration to fit the continuum; range: [1,20]. [5]"))

        paramList.append(reflex.RecipeParameter(recipe = self.recipe_name,
            displayName = "det_line_thres",   group = "Fitting", description =
         "Normalized threshold for line acceptance; range: [0,0.5]. [0.02]"))

        paramList.append(reflex.RecipeParameter(recipe = self.recipe_name,
            displayName = "det_line_resol",   group = "Thresholds", description =
         "Minimum accepted separation between two lines (in [nm]) "\
         "used for the multi  gaussian fits. It is also used to identify "\
                      "the lines to be measured;range: [0.001,0.1]. [0.01]"))

        paramList.append(reflex.RecipeParameter(recipe = self.recipe_name,
            displayName = "det_line_smwidth",   group = "Fitting", description =
                                                "Number of pixels in the boxcar to be averaged to fit the continuum; useful to deal with noisy spectrum; range: [1,50]. [4]"))


        paramList.append(reflex.RecipeParameter(recipe = self.recipe_name,
            displayName = "fit_ngauss_width",   group = "Fitting", description =
                                                "Initial guess for the parameter related with the width of the Gaussian [mA]; 0.5 [0.1,2.5]"))

        return paramList

    def readFitsData(self, fitsFiles):
        self.wavel_range = list()
        self.frames = dict()
        for frame in fitsFiles:
            if frame == '':
                continue
            category = frame.category
            self.frames[category] = frame

        self.eqw  = PlotableData(self.frames["LINE_EQWIDTH"], marker=".", size=50, mode="scatter", x="WAVEL", y = "EW")
#        self.waves_line  = PlotableData2(self.frames["LINE_EQWIDTH"], marker=".", size=50, mode="scatter", x="WAVEL", y = "EW")
        self.waves_line = PlotArrayRow(self.frames["LINE_EQWIDTH"], iline=self.line_index)
#        self.waves_line = PlotableData(self.frames["LINE_EQWIDTH"], marker=".", size=50, mode="scatter", x="WAVEL_LOC", y = "FLUX_LOC")
#        self.waves_line = PlotableData(self.frames["LINE_EQWIDTH"], marker=".", size=50, mode="scatter", x="WAVEL_LOC", y = "FLUX_LOC")
#        hdu = fits.open(self.frames["LINE_EQWIDTH"])
#        self.waveloc = hdu[1].data['WAVEL_LOC'][iline]
#        self.fluxloc = hdu[1].data['FLUX_LOC'][iline]
#        hdu.close()



    def addSubplots(self, figure):
#        gs_ls = gridspec.GridSpec(1, 1, bottom = 0.45, top = 0.95)
#        gs_eqw  = gridspec.GridSpec(1, 1, bottom = 0.05, top = 0.35)
#        self.subplot_eqw = figure.add_subplot(gs_eqw[0,0])
#        self.subplot_ls = figure.add_subplot(gs_ls[0,0])
        self.subplot_ls = figure.add_subplot(2,1,1)
        self.subplot_eqw = figure.add_subplot(2,1,2)
        self.subplot_ls.plot(self.waves_line.wave_l, self.waves_line.spec_l)
        self.subplot_ls.plot(self.waves_line.wave_l, self.waves_line.fit_l)
        self.line_selector = figure.add_axes([0.20, 0.5, 0.60, 0.02])

    def plotProductsGraphics(self):
        self.plotSpec()
        self.firstClick = False

    def plotSpec(self):
        eqw_title   = "Equivalent Width"
        eqw_tooltip = "Blue: equivalent width as function of wavelength.\n" 
        eqw_y_label = r"Equivalent Width "
        all_x_label = "Wavelength [nm]"

        self.eqw.plot(self.subplot_eqw)
        self.eqw.plot(self.subplot_eqw, title = eqw_title,
            tooltip = eqw_tooltip, x_label = all_x_label,
            y_label = eqw_y_label, bars = self.wavel_range)

    def plotWidgets(self):
        widgets = list()
        self.clicks = 0
        self.clickable_eqw = InteractiveClickableSubplot(self.subplot_eqw, self.wavelRange)
        widgets.append(self.clickable_eqw)
        nlines = self.waves_line.nlines
        self.slider =espda_discrete_slider.InteractiveSlider2(self.line_selector,self.setLineSelectCallBack,0,nlines,self.line_index,1,"line index")
        widgets.append(self.slider)

        return widgets


    def setLineSelectCallBack(self, index) :        
        self.line_index = int(index)
        self.subplot_ls.cla()
        self.waves_line = PlotArrayRow(self.frames["LINE_EQWIDTH"], iline=self.line_index)
        self.subplot_ls.plot(self.waves_line.wave_l, self.waves_line.spec_l)
        self.subplot_ls.plot(self.waves_line.wave_l, self.waves_line.fit_l)


    def wavelRange(self, point):
        ### I tried to do something with the clickable subplot, but does not work...
        self.clicks = self.clicks + 1
        index = self.eqw.getXIndex(point.xdata)
        self.line_index = int(index)
        self.subplot_ls.cla()
        self.waves_line = PlotArrayRow(self.frames["LINE_EQWIDTH"], iline=self.line_index)
        self.subplot_ls.plot(self.waves_line.wave_l, self.waves_line.spec_l)
        self.subplot_ls.plot(self.waves_line.wave_l, self.waves_line.fit_l)

        


    def setWindowTitle(self):
      title = 'ESPRESSO Data Analysis Software: Equivalent Width Computation'
      return title

if __name__ == '__main__':
    import reflex_interactive_app
    import sys
    import pkg_resources
    from espda_plot_common import *
    interactive_app = reflex_interactive_app.PipelineInteractiveApp(
        enable_init_sop = True)
    interactive_app.parse_args()
    if not import_success:
        interactive_app.setEnableGUI(False)
    if interactive_app.isGUIEnabled():
        dataPlotManager = DataPlotterManager()
        interactive_app.setPlotManager(dataPlotManager)
        interactive_app.showGUI()
    else:
        interactive_app.set_continue_mode()
    interactive_app.print_outputs()
    sys.exit()
